#ifndef __XREADER__
#define __XREADER__

#include <fstream.h>
#include <iostream.h>
#include <tricks.h>

struct vertex
{
   float x,y,z;
   float xn,yn,zn;

   float u,v;

   int px,py;
};

struct face
{
   int p1,p2,p3;
   float xn,yn,zn;
   float z;
};

struct obj
{
   int vertices, faces;
   vertex *v;
   face *f;
};

void readobj( obj *O, char *file )
{
   ifstream fin( file, ios::in );
   char c;
   char s[80];
   int test;

   O->vertices = 0;
   O->faces = 0;
   O->v = NULL;
   O->f = NULL;

   if( fin.fail() ) {
      cout << "Unable to open " << file << endl;
      exit(1);
   }

   for( int i=1; i<10; i++ )
      fin.getline( s, 80 );

   fin >> O->vertices >> c;

   if( fin.fail() ) {
      cout << "Invalid object!\n";
      exit(1);
   }

   O->v = new vertex[ O->vertices ];

   for( i=0; i<O->vertices; i++ )
      fin >> O->v[i].x >> c >> O->v[i].y >> c >> O->v[i].z >> c >> c;

   fin.getline( s, 80 );  
   fin >> O->faces >> c;

   if( fin.fail() ) {
      cout << "Invalid object!\n";
      exit(1);
   }

   O->f = new face[ O->faces ];

   for( i=0; i<O->faces; i++ ) {
      fin >> test >> c >> O->f[i].p1 >> c;
      fin >> O->f[i].p2 >> c >> O->f[i].p3 >> c >> c;

      if( test != 3 ) {
         cout << "Only 3-point faces are allowed!\n\n";
         exit(1);
      }
   }
      
   while( c != 'N' )          // go to Mesh_N_ormals
      fin.get( c );

   fin.getline( s, 80 );
   fin.getline( s, 80 );

   for( i=0; i<O->vertices; i++ )
      fin >> O->v[i].xn >> c >> O->v[i].yn >> c >> O->v[i].zn >> c >> c;

   // calculate face normals

   for( i=0; i<O->faces; i++ ) {

      float p1x = O->v[O->f[i].p1].x;
      float p1y = O->v[O->f[i].p1].y;
      float p1z = O->v[O->f[i].p1].z;

      float p2x = O->v[O->f[i].p2].x;
      float p2y = O->v[O->f[i].p2].y;
      float p2z = O->v[O->f[i].p2].z;

      float p3x = O->v[O->f[i].p3].x;
      float p3y = O->v[O->f[i].p3].y;
      float p3z = O->v[O->f[i].p3].z;

      O->f[i].xn = ((p3y - p2y) * (p1z - p2z))
                 - ((p1y - p2y) * (p3z - p2z));

      O->f[i].yn = ((p3z - p2z) * (p1x - p2x))
                 - ((p1z - p2z) * (p3x - p2x));

      O->f[i].zn = ((p3y - p2y) * (p1x - p2x))
                 - ((p1y - p2y) * (p3x - p2x));

      O->f[i].z  = (p1z+p2z+p3z)/3;
   }

   return;
}

void rotate( const obj *O, float x, float y, float z, float zd, float p )
{
   // precalc values..

   float sx = sin(x);
   float sy = sin(y);
   float sz = sin(z);

   float cx = cos(x);
   float cy = cos(y);
   float cz = cos(z);

   float mx1 = sx*sy*sz + cz*cx;
   float my1 = sx*cy;
   float mz1 = sz*cx - sx*sy*cz;

   float mx2 = cx*sy*sz - cz*sx;
   float my2 = cx*cy;
   float mz2 = -cz*sy*cx - sz*sx;

   float mx3 = -sz*cy;
   float my3 = sy;
   float mz3 = cz*cy;

   for( int i=0; i<O->vertices; i++ )
   {
      // rotate coords

      x = O->v[i].x;
      y = O->v[i].y;
      z = O->v[i].z;

      O->v[i].x = x*mx1 + y*my1 + z*mz1;
      O->v[i].y = x*mx2 + y*my2 + z*mz2;
      O->v[i].z = x*mx3 + y*my3 + z*mz3;

      // project

      z += zd;
      float zi = 1/z;

      O->v[i].px = (QuickFist((x*zi)*p) + 160)<<8;
      O->v[i].py = QuickFist((y*zi)*p*.8) + 100;

      // rotate normals

      x = O->v[i].xn;
      y = O->v[i].yn;
      z = O->v[i].zn;

      O->v[i].xn = x*mx1 + y*my1 + z*mz1;
      O->v[i].yn = x*mx2 + y*my2 + z*mz2;
      O->v[i].zn = x*mx3 + y*my3 + z*mz3;
   }

   for( i=0; i<O->faces; i++) {
      float p1x = O->v[O->f[i].p1].x;
      float p1y = O->v[O->f[i].p1].y;
      float p1z = O->v[O->f[i].p1].z;

      float p2x = O->v[O->f[i].p2].x;
      float p2y = O->v[O->f[i].p2].y;
      float p2z = O->v[O->f[i].p2].z;

      float p3x = O->v[O->f[i].p3].x;
      float p3y = O->v[O->f[i].p3].y;
      float p3z = O->v[O->f[i].p3].z;
/*
      O->f[i].xn = ((p3y - p2y) * (p1z - p2z))
                 - ((p1y - p2y) * (p3z - p2z));

      O->f[i].yn = ((p3z - p2z) * (p1x - p2x))
                 - ((p1z - p2z) * (p3x - p2x));
*/
      O->f[i].zn = ((p3y - p2y) * (p1x - p2x))
                 - ((p1y - p2y) * (p3x - p2x));

      O->f[i].z  = (p1z+p2z+p3z)*0.333333333333333333333333;
   }
}


void rotateHi( const obj *O, float x, float y, float z, float zd, float p )
{
   // precalc values..

   float sx = sin(x);
   float sy = sin(y);
   float sz = sin(z);

   float cx = cos(x);
   float cy = cos(y);
   float cz = cos(z);

   float mx1 = sx*sy*sz + cz*cx;
   float my1 = sx*cy;
   float mz1 = sz*cx - sx*sy*cz;

   float mx2 = cx*sy*sz - cz*sx;
   float my2 = cx*cy;
   float mz2 = -cz*sy*cx - sz*sx;

   float mx3 = -sz*cy;
   float my3 = sy;
   float mz3 = cz*cy;

   for( int i=0; i<O->vertices; i++ )
   {
      // rotate coords

      x = O->v[i].x;
      y = O->v[i].y;
      z = O->v[i].z;

      O->v[i].x = x*mx1 + y*my1 + z*mz1;
      O->v[i].y = x*mx2 + y*my2 + z*mz2;
      O->v[i].z = x*mx3 + y*my3 + z*mz3;

      // project

      z += zd;
      float zi = 1/z;

      O->v[i].px = (QuickFist((x*zi)*p) + 320)<<8;
      O->v[i].py = QuickFist((y*zi)*p) + 240;

      // rotate normals

      x = O->v[i].xn;
      y = O->v[i].yn;
      z = O->v[i].zn;

      O->v[i].xn = x*mx1 + y*my1 + z*mz1;
      O->v[i].yn = x*mx2 + y*my2 + z*mz2;
      O->v[i].zn = x*mx3 + y*my3 + z*mz3;
   }

   for( i=0; i<O->faces; i++) {
      float p1x = O->v[O->f[i].p1].px;
      float p1y = O->v[O->f[i].p1].py;
      float p1z = O->v[O->f[i].p1].z;

      float p2x = O->v[O->f[i].p2].px;
      float p2y = O->v[O->f[i].p2].py;
      float p2z = O->v[O->f[i].p2].z;

      float p3x = O->v[O->f[i].p3].px;
      float p3y = O->v[O->f[i].p3].py;
      float p3z = O->v[O->f[i].p3].z;
/*
      O->f[i].xn = ((p3y - p2y) * (p1z - p2z))
                 - ((p1y - p2y) * (p3z - p2z));

      O->f[i].yn = ((p3z - p2z) * (p1x - p2x))
                 - ((p1z - p2z) * (p3x - p2x));
*/
      O->f[i].zn = ((p3y - p2y) * (p1x - p2x))
                 - ((p1y - p2y) * (p3x - p2x));

      O->f[i].z  = (p1z+p2z+p3z);
   }
}

#endif
